/*
 * Copyright (C) 2025 Carmix <carmixdev@gmail.com>
 *
 * This file is part of HexWalk.
 *
 * HexWalk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HexWalk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef RESULTTYPE_H
#define RESULTTYPE_H
#include <QString>
#include <qglobal.h>

typedef struct Result_S{
    qint64 cursor;
    QString hexdatastr;
    QString datastr;

}Result_S;

typedef struct BinwalkResult_S{
    qint64 cursor;
    QString datastr;

}BinwalkResult_S;

#endif // RESULTTYPE_H
